
/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "bgnet.h"

int Bgnet::Connect( const char *host, int port )
{
	int fd;
	struct hostent *he;
	struct sockaddr_in their_addr; // connector's address information 

	if( ( he = gethostbyname( host ) ) == NULL ) // get the host info 
	{
		perror( "gethostbyname" );
		return ( -1 );
	}

	if( ( fd = socket( AF_INET, SOCK_STREAM, 0 ) ) == -1 )
	{
		perror( "socket" );
		return ( -1 );
	}

	their_addr.sin_family = AF_INET; // host byte order 
	their_addr.sin_port = htons( port ); // short, network byte order 
	their_addr.sin_addr = *( ( struct in_addr * )he->h_addr );
	memset( their_addr.sin_zero, '\0', sizeof their_addr.sin_zero );

	if( connect( fd, ( struct sockaddr * )&their_addr, sizeof their_addr ) == -1 )
	{
		perror( "connect" );
		return ( -1 );
	}

	return fd;
}

void Bgnet::Disconnect( int fd )
{
	close( fd );
}

int Bgnet::SendAll( int fd, const char *data, int data_length )
{
	int bytes_sent = 0;
	int result;

	while( bytes_sent < data_length )
	{
		result = send( fd, data + bytes_sent, data_length - bytes_sent, 0 );
		if( result == -1 )
		{
			if( errno == EAGAIN )
				continue; // optionally, you may also want to check if errno == EINTR
			bytes_sent = -1;
		}
		bytes_sent += result;
	}
	return bytes_sent;
}

int Bgnet::Select( int fd )
{
	struct timeval tv;
	fd_set readfds;

	tv.tv_sec = 2;
	tv.tv_usec = 500000;

	FD_ZERO( &readfds );
	FD_SET( fd, &readfds );

	// don't care about writefds and exceptfds:
	select( fd + 1, &readfds, NULL, NULL, &tv );

	if( FD_ISSET( fd, &readfds ) )
		return 1;
	else
		return 0;
}

int Bgnet::Receive( int fd, char *buf, int len )
{
	int numbytes;

	bzero( buf, len );
	if( ( numbytes = recv( fd, buf, len - 1, 0 ) ) == -1 )
	{
		perror( "recv" );
		return ( -1 );
	}

	buf[numbytes] = '\0';

	//printf( "Received: %s", buf );

	return numbytes;
}

int Bgnet::ReadLine( int fd, char *buf, size_t len )
{
	unsigned int pos = 0;
	ssize_t bytes = 0;

	char *ch = ( char * )malloc( sizeof( char ) );

	while( ( ch[0] != '\n' ) && ( pos < len ) )
	{
		if( ( bytes = recv( fd, ch, 1, 0 ) ) == -1 )
		{
			perror( strerror( errno ) );
			return -1;
		}
		buf[pos++] = ch[0];
	}
	buf[pos++] = '\0';

	return bytes;
}
